! ##################################################################################################################################
! Begin MIT license text.                                                                                    
! _______________________________________________________________________________________________________
                                                                                                         
! Copyright 2019 Dr William R Case, Jr (dbcase29@gmail.com)                                              
                                                                                                         
! Permission is hereby granted, free of charge, to any person obtaining a copy of this software and      
! associated documentation files (the "Software"), to deal in the Software without restriction, including
! without limitation the rights to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
! copies of the Software, and to permit persons to whom the Software is furnished to do so, subject to   
! the following conditions:                                                                              
                                                                                                         
! The above copyright notice and this permission notice shall be included in all copies or substantial   
! portions of the Software and documentation.                                                                              
                                                                                                         
! THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS                                
! OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,                            
! FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE                            
! AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER                                 
! LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,                          
! OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN                              
! THE SOFTWARE.                                                                                          
! _______________________________________________________________________________________________________
                                                                                                        
! End MIT license text.                                                                                      

      MODULE ALLOCATED_ARRAY_DATA

! This module is only used for checking whether arrays are allocated. It is not a module needed in the normal processing of MYSTRAN.
! The check on whether arrays are allocated or not can be done by including a Bulk Data DEBUG entry (see MYSTRAN User's Reference
! Manual for which DEBUG number to use)

      USE PENTIUM_II_KIND, ONLY     :  BYTE, LONG, DOUBLE
      USE CONSTANTS_1, ONLY         :  ZERO

      IMPLICIT NONE

      SAVE

      INTEGER(LONG), PRIVATE        :: II
      INTEGER(LONG), PARAMETER      :: NUM_ALLOC_ARRAYS = 585

      CHARACTER( 31*BYTE)           :: ALLOCATED_ARRAY_NAMES(NUM_ALLOC_ARRAYS)= (/'ABAND                          ', & ! NAME(  1)
                                                                                  'AGRID                          ', & ! NAME(  2)
                                                                                  'ALG                            ', & ! NAME(  3)
                                                                                  'ALL_SETS_ARRAY                 ', & ! NAME(  4)
                                                                                  'AROW                           ', & ! NAME(  5)
                                                                                  'BAROFF                         ', & ! NAME(  6)
                                                                                  'BUSHOFF                        ', & ! NAME(  7)
                                                                                  'BBAND                          ', & ! NAME(  8)
                                                                                  'BE1                            ', & ! NAME(  9)
                                                                                  'BE2                            ', & ! NAME( 10)
                                                                                  'BE3                            ', & ! NAME( 11)
                                                                                  'BGRID                          ', & ! NAME( 12)
                                                                                  'CCS1                           ', & ! NAME( 13)
                                                                                  'CCS2                           ', & ! NAME( 14)
                                                                                  'CCS3                           ', & ! NAME( 15)
                                                                                  'CETEMP                         ', & ! NAME( 16)
                                                                                  'CG_LTM                         ', & ! NAME( 17)
                                                                                  'CGTEMP                         ', & ! NAME( 18)
                                                                                  'CMASS                          ', & ! NAME( 19)
                                                                                  'CONM2                          ', & ! NAME( 20)
                                                                                  'CORD                           ', & ! NAME( 21)
                                                                                  'CROW                           ', & ! NAME( 22)
                                                                                  'CRS1                           ', & ! NAME( 23)
                                                                                  'CRS2                           ', & ! NAME( 24)
                                                                                  'CRS3                           ', & ! NAME( 25)
                                                                                  'DLR                            ', & ! NAME( 26)
                                                                                  'DLRt                           ', & ! NAME( 27)
                                                                                  'DOFPIN                         ', & ! NAME( 28)
                                                                                  'DT                             ', & ! NAME( 29)
                                                                                  'DUM1                           ', & ! NAME( 30)
                                                                                  'DUM2                           ', & ! NAME( 31)
                                                                                  'DUM3                           ', & ! NAME( 32)
                                                                                  'EDAT                           ', & ! NAME( 33)
                                                                                  'EID_OUT_ARRAY                  ', & ! NAME( 34)
                                                                                  'EIGEN_VAL                      ', & ! NAME( 35)
                                                                                  'EIGEN_VEC                      ', & ! NAME( 36)
                                                                                  'ELDT                           ', & ! NAME( 37)
                                                                                  'ELOUT                          ', & ! NAME( 38)
                                                                                  'EMS                            ', & ! NAME( 39)
                                                                                  'EMSCOL                         ', & ! NAME( 40)
                                                                                  'EMSKEY                         ', & ! NAME( 41)
                                                                                  'EMSPNT                         ', & ! NAME( 42)
                                                                                  'EOFF                           ', & ! NAME( 43)
                                                                                  'EPNT                           ', & ! NAME( 44)
                                                                                  'ESORT1                         ', & ! NAME( 45)
                                                                                  'ESORT2                         ', & ! NAME( 46)
                                                                                  'ETEMP                          ', & ! NAME( 47)
                                                                                  'ETYPE                          ', & ! NAME( 48)
                                                                                  'FA_COL                         ', & ! NAME( 49)
                                                                                  'FEMAP_EL_NUMS                  ', & ! NAME( 50)
                                                                                  'FEMAP_EL_VECS                  ', & ! NAME( 51)
                                                                                  'FF_COL                         ', & ! NAME( 52)
                                                                                  'FG_COL                         ', & ! NAME( 53)
                                                                                  'FL_COL                         ', & ! NAME( 54)
                                                                                  'FM_COL                         ', & ! NAME( 55)
                                                                                  'FN_COL                         ', & ! NAME( 56)
                                                                                  'FO_COL                         ', & ! NAME( 57)
                                                                                  'FORMOM_SIDS                    ', & ! NAME( 58)
                                                                                  'FR_COL                         ', & ! NAME( 59)
                                                                                  'FS_COL                         ', & ! NAME( 60)
                                                                                  'FTNAME                         ', & ! NAME( 61)
                                                                                  'GEN_MASS                       ', & ! NAME( 62)
                                                                                  'GID_OUT_ARRAY                  ', & ! NAME( 63)
                                                                                  'GMN                            ', & ! NAME( 64)
                                                                                  'GMN_FULL                       ', & ! NAME( 65)
                                                                                  'GMNt                           ', & ! NAME( 66)
                                                                                  'GMNt_FULL                      ', & ! NAME( 67)
                                                                                  'GOA                            ', & ! NAME( 68)
                                                                                  'GOA_FULL                       ', & ! NAME( 69)
                                                                                  'GOAt                           ', & ! NAME( 70)
                                                                                  'GOAt_FULL                      ', & ! NAME( 71)
                                                                                  'GRAV_SIDS                      ', & ! NAME( 72)
                                                                                  'GRID                           ', & ! NAME( 73)
                                                                                  'GRID_ELEM_CONN_ARRAY           ', & ! NAME( 74)
                                                                                  'GRID_ID                        ', & ! NAME( 75)
                                                                                  'GRID_SEQ                       ', & ! NAME( 76)
                                                                                  'GROUT                          ', & ! NAME( 77)
                                                                                  'GTEMP                          ', & ! NAME( 78)
                                                                                  'HMN                            ', & ! NAME( 79)
                                                                                  'HMN_FULL                       ', & ! NAME( 80)
                                                                                  'I_CCS1                         ', & ! NAME( 81)
                                                                                  'I_CCS2                         ', & ! NAME( 82)
                                                                                  'I_CCS3                         ', & ! NAME( 83)
                                                                                  'I_CG_LTM                       ', & ! NAME( 84)
                                                                                  'I_CRS1                         ', & ! NAME( 85)
                                                                                  'I_CRS2                         ', & ! NAME( 86)
                                                                                  'I_CRS3                         ', & ! NAME( 87)
                                                                                  'I_DLR                          ', & ! NAME( 88)
                                                                                  'I_DLRt                         ', & ! NAME( 89)
                                                                                  'I_GMN                          ', & ! NAME( 90)
                                                                                  'I_GMNt                         ', & ! NAME( 91)
                                                                                  'I_GOA                          ', & ! NAME( 92)
                                                                                  'I_GOAt                         ', & ! NAME( 93)
                                                                                  'I_HMN                          ', & ! NAME( 94)
                                                                                  'I_IF_LTM                       ', & ! NAME( 95)
                                                                                  'I_IRR                          ', & ! NAME( 96)
                                                                                  'I_KAA                          ', & ! NAME( 97)
                                                                                  'I_KAO                          ', & ! NAME( 98)
                                                                                  'I_KFF                          ', & ! NAME( 99)
                                                                                  'I_KFS                          ', & ! NAME(100)
                                                                                  'I_KFSe                         ', & ! NAME(101)
                                                                                  'I_KGG                          ', & ! NAME(102)
                                                                                  'I_KLL                          ', & ! NAME(103)
                                                                                  'I_KLLs                         ', & ! NAME(104)
                                                                                  'I_KMM                          ', & ! NAME(105)
                                                                                  'I_KMN                          ', & ! NAME(106)
                                                                                  'I_KMSM                         ', & ! NAME(107)
                                                                                  'I_KMSMn                        ', & ! NAME(108)
                                                                                  'I_KMSMs                        ', & ! NAME(109)
                                                                                  'I_KNM                          ', & ! NAME(110)
                                                                                  'I_KNN                          ', & ! NAME(111)
                                                                                  'I_KOO                          ', & ! NAME(112)
                                                                                  'I_KOOs                         ', & ! NAME(113)
                                                                                  'I_KRL                          ', & ! NAME(114)
                                                                                  'I_KRR                          ', & ! NAME(115)
                                                                                  'I_KRRcb                        ', & ! NAME(116)
                                                                                  'I_KRRcbn                       ', & ! NAME(117)
                                                                                  'I_KRRcbs                       ', & ! NAME(118)
                                                                                  'I_KSF                          ', & ! NAME(119)
                                                                                  'I_KSFD                         ', & ! NAME(120)
                                                                                  'I_KSS                          ', & ! NAME(121)
                                                                                  'I_KSSe                         ', & ! NAME(122)
                                                                                  'I_KXX                          ', & ! NAME(123)
                                                                                  'I_LMN                          ', & ! NAME(124)
                                                                                  'I_LTM                          ', & ! NAME(125)
                                                                                  'I_MAA                          ', & ! NAME(126)
                                                                                  'I_MAO                          ', & ! NAME(127)
                                                                                  'I_MFF                          ', & ! NAME(128)
                                                                                  'I_MFS                          ', & ! NAME(129)
                                                                                  'I_MGG                          ', & ! NAME(130)
                                                                                  'I_MGGC                         ', & ! NAME(131)
                                                                                  'I_MGGE                         ', & ! NAME(132)
                                                                                  'I_MGGS                         ', & ! NAME(133)
                                                                                  'I_MLL                          ', & ! NAME(134)
                                                                                  'I_MLLn                         ', & ! NAME(135)
                                                                                  'I_MLLs                         ', & ! NAME(136)
                                                                                  'I_MLR                          ', & ! NAME(137)
                                                                                  'I_MMM                          ', & ! NAME(138)
                                                                                  'I_MMN                          ', & ! NAME(139)
                                                                                  'I_MNM                          ', & ! NAME(140)
                                                                                  'I_MNN                          ', & ! NAME(141)
                                                                                  'I_MOO                          ', & ! NAME(142)
                                                                                  'I_MPF0                         ', & ! NAME(143)
                                                                                  'I_MRL                          ', & ! NAME(144)
                                                                                  'I_MRN                          ', & ! NAME(145)
                                                                                  'I_MRR                          ', & ! NAME(146)
                                                                                  'I_MRRcb                        ', & ! NAME(147)
                                                                                  'I_MRRcbn                       ', & ! NAME(148)
                                                                                  'I_MSF                          ', & ! NAME(149)
                                                                                  'I_MSS                          ', & ! NAME(150)
                                                                                  'I_MXX                          ', & ! NAME(151)
                                                                                  'I_MXXn                         ', & ! NAME(152)
                                                                                  'I_PA                           ', & ! NAME(153)
                                                                                  'I_PF                           ', & ! NAME(154)
                                                                                  'I_PF_TMP                       ', & ! NAME(155)
                                                                                  'I_PFYS                         ', & ! NAME(156)
                                                                                  'I_PFYS1                        ', & ! NAME(157)
                                                                                  'I_PG                           ', & ! NAME(158)
                                                                                  'I_PHIXA                        ', & ! NAME(159)
                                                                                  'I_PHIZL                        ', & ! NAME(160)
                                                                                  'I_PHIZL1                       ', & ! NAME(161)
                                                                                  'I_PHIZL1t                      ', & ! NAME(162)
                                                                                  'I_PHIZL2                       ', & ! NAME(163)
                                                                                  'I_PL                           ', & ! NAME(164)
                                                                                  'I_PM                           ', & ! NAME(165)
                                                                                  'I_PN                           ', & ! NAME(166)
                                                                                  'I_PO                           ', & ! NAME(167)
                                                                                  'I_PR                           ', & ! NAME(168)
                                                                                  'I_PS                           ', & ! NAME(169)
                                                                                  'I_QSYS                         ', & ! NAME(170)
                                                                                  'I_RMG                          ', & ! NAME(171)
                                                                                  'I_RMM                          ', & ! NAME(172)
                                                                                  'I_RMN                          ', & ! NAME(173)
                                                                                  'I2_DLR                         ', & ! NAME(174)
                                                                                  'I2_DLRt                        ', & ! NAME(175)
                                                                                  'I2_GMN                         ', & ! NAME(176)
                                                                                  'I2_GOA                         ', & ! NAME(177)
                                                                                  'I2_KLL                         ', & ! NAME(178)
                                                                                  'I2_KLLs                        ', & ! NAME(179)
                                                                                  'I2_KMSM                        ', & ! NAME(180)
                                                                                  'I2_KMSMs                       ', & ! NAME(181)
                                                                                  'I2_KOO                         ', & ! NAME(182)
                                                                                  'I2_KOOs                        ', & ! NAME(183)
                                                                                  'I2_MGG                         ', & ! NAME(184)
                                                                                  'I2_MLL                         ', & ! NAME(185)
                                                                                  'I2_MLLs                        ', & ! NAME(186)
                                                                                  'I2_PHIZL1                      ', & ! NAME(187)
                                                                                  'I2_PHIZL1t                     ', & ! NAME(188)
                                                                                  'IF_LTM                         ', & ! NAME(189)
                                                                                  'IN4_COL_MAP                    ', & ! NAME(190)
                                                                                  'IN4_MAT                        ', & ! NAME(191)
                                                                                  'IN4FIL                         ', & ! NAME(192)
                                                                                  'IN4FIL_NUM                     ', & ! NAME(193)
                                                                                  'INV_GRID_SEQ                   ', & ! NAME(194)
                                                                                  'IRR                            ', & ! NAME(195)
                                                                                  'IWORK                          ', & ! NAME(196)
                                                                                  'J_AROW                         ', & ! NAME(197)
                                                                                  'J_CCS1                         ', & ! NAME(198)
                                                                                  'J_CCS2                         ', & ! NAME(199)
                                                                                  'J_CCS3                         ', & ! NAME(200)
                                                                                  'J_CG_LTM                       ', & ! NAME(201)
                                                                                  'J_CRS1                         ', & ! NAME(202)
                                                                                  'J_CRS2                         ', & ! NAME(203)
                                                                                  'J_CRS3                         ', & ! NAME(204)
                                                                                  'J_DLR                          ', & ! NAME(205)
                                                                                  'J_DLRt                         ', & ! NAME(206)
                                                                                  'J_GMN                          ', & ! NAME(207)
                                                                                  'J_GMNt                         ', & ! NAME(208)
                                                                                  'J_GOA                          ', & ! NAME(209)
                                                                                  'J_GOAt                         ', & ! NAME(210)
                                                                                  'J_HMN                          ', & ! NAME(211)
                                                                                  'J_IF_LTM                       ', & ! NAME(212)
                                                                                  'J_IRR                          ', & ! NAME(213)
                                                                                  'J_KAA                          ', & ! NAME(214)
                                                                                  'J_KAO                          ', & ! NAME(215)
                                                                                  'J_KFF                          ', & ! NAME(216)
                                                                                  'J_KFS                          ', & ! NAME(217)
                                                                                  'J_KFSe                         ', & ! NAME(218)
                                                                                  'J_KGG                          ', & ! NAME(219)
                                                                                  'J_KLL                          ', & ! NAME(220)
                                                                                  'J_KLLs                         ', & ! NAME(221)
                                                                                  'J_KMM                          ', & ! NAME(222)
                                                                                  'J_KMN                          ', & ! NAME(223)
                                                                                  'J_KMSM                         ', & ! NAME(224)
                                                                                  'J_KMSMn                        ', & ! NAME(225)
                                                                                  'J_KMSMs                        ', & ! NAME(226)
                                                                                  'J_KNM                          ', & ! NAME(227)
                                                                                  'J_KNN                          ', & ! NAME(228)
                                                                                  'J_KOO                          ', & ! NAME(229)
                                                                                  'J_KOOs                         ', & ! NAME(230)
                                                                                  'J_KRL                          ', & ! NAME(231)
                                                                                  'J_KRR                          ', & ! NAME(232)
                                                                                  'J_KRRcb                        ', & ! NAME(233)
                                                                                  'J_KRRcbn                       ', & ! NAME(234)
                                                                                  'J_KRRcbs                       ', & ! NAME(235)
                                                                                  'J_KSF                          ', & ! NAME(236)
                                                                                  'J_KSFD                         ', & ! NAME(237)
                                                                                  'J_KSS                          ', & ! NAME(238)
                                                                                  'J_KSSe                         ', & ! NAME(239)
                                                                                  'J_KXX                          ', & ! NAME(240)
                                                                                  'J_LMN                          ', & ! NAME(241)
                                                                                  'J_LTM                          ', & ! NAME(242)
                                                                                  'J_MAA                          ', & ! NAME(243)
                                                                                  'J_MAO                          ', & ! NAME(244)
                                                                                  'J_MFF                          ', & ! NAME(245)
                                                                                  'J_MFS                          ', & ! NAME(246)
                                                                                  'J_MGG                          ', & ! NAME(247)
                                                                                  'J_MGGC                         ', & ! NAME(248)
                                                                                  'J_MGGE                         ', & ! NAME(249)
                                                                                  'J_MGGS                         ', & ! NAME(250)
                                                                                  'J_MLL                          ', & ! NAME(251)
                                                                                  'J_MLLn                         ', & ! NAME(252)
                                                                                  'J_MLLs                         ', & ! NAME(253)
                                                                                  'J_MLR                          ', & ! NAME(254)
                                                                                  'J_MMM                          ', & ! NAME(255)
                                                                                  'J_MMN                          ', & ! NAME(256)
                                                                                  'J_MNM                          ', & ! NAME(257)
                                                                                  'J_MNN                          ', & ! NAME(258)
                                                                                  'J_MOO                          ', & ! NAME(2594
                                                                                  'J_MPF0                         ', & ! NAME(260)
                                                                                  'J_MRL                          ', & ! NAME(261)
                                                                                  'J_MRN                          ', & ! NAME(262)
                                                                                  'J_MRR                          ', & ! NAME(263)
                                                                                  'J_MRRcb                        ', & ! NAME(264)
                                                                                  'J_MRRcbn                       ', & ! NAME(265)
                                                                                  'J_MSF                          ', & ! NAME(266)
                                                                                  'J_MSS                          ', & ! NAME(267)
                                                                                  'J_MXX                          ', & ! NAME(268)
                                                                                  'J_MXXn                         ', & ! NAME(269)
                                                                                  'J_PA                           ', & ! NAME(270)
                                                                                  'J_PF                           ', & ! NAME(271)
                                                                                  'J_PF_TMP                       ', & ! NAME(272)
                                                                                  'J_PFYS                         ', & ! NAME(273)
                                                                                  'J_PFYS1                        ', & ! NAME(274)
                                                                                  'J_PG                           ', & ! NAME(275)
                                                                                  'J_PHIXA                        ', & ! NAME(276)
                                                                                  'J_PHIZL                        ', & ! NAME(277)
                                                                                  'J_PHIZL1                       ', & ! NAME(278)
                                                                                  'J_PHIZL1t                      ', & ! NAME(279)
                                                                                  'J_PHIZL2                       ', & ! NAME(280)
                                                                                  'J_PL                           ', & ! NAME(281)
                                                                                  'J_PM                           ', & ! NAME(282)
                                                                                  'J_PN                           ', & ! NAME(283)
                                                                                  'J_PO                           ', & ! NAME(284)
                                                                                  'J_PR                           ', & ! NAME(285)
                                                                                  'J_PS                           ', & ! NAME(286)
                                                                                  'J_QSYS                         ', & ! NAME(287)
                                                                                  'J_RMG                          ', & ! NAME(288)
                                                                                  'J_RMM                          ', & ! NAME(289)
                                                                                  'J_RMN                          ', & ! NAME(290)
                                                                                  'KAA                            ', & ! NAME(291)
                                                                                  'KAA_FULL                       ', & ! NAME(292)
                                                                                  'KAO                            ', & ! NAME(293)
                                                                                  'KAO_FULL                       ', & ! NAME(294)
                                                                                  'KE                             ', & ! NAME(295)
                                                                                  'KED                            ', & ! NAME(296)
                                                                                  'KEM                            ', & ! NAME(297)
                                                                                  'KFF                            ', & ! NAME(298)
                                                                                  'KFF_FULL                       ', & ! NAME(299)
                                                                                  'KFS                            ', & ! NAME(300)
                                                                                  'KFS_FULL                       ', & ! NAME(301)
                                                                                  'KFSe                           ', & ! NAME(302)
                                                                                  'KFSe_FULL                      ', & ! NAME(303)
                                                                                  'KGG                            ', & ! NAME(304)
                                                                                  'KLL                            ', & ! NAME(305)
                                                                                  'KLLs                           ', & ! NAME(306)
                                                                                  'KMM                            ', & ! NAME(307)
                                                                                  'KMM_FULL                       ', & ! NAME(308)
                                                                                  'KMN                            ', & ! NAME(309)
                                                                                  'KMSM                           ', & ! NAME(310)
                                                                                  'KMSMn                          ', & ! NAME(311)
                                                                                  'KMSMs                          ', & ! NAME(312)
                                                                                  'KNM                            ', & ! NAME(313)
                                                                                  'KNM_FULL                       ', & ! NAME(314)
                                                                                  'KNN                            ', & ! NAME(315)
                                                                                  'KNN_FULL                       ', & ! NAME(316)
                                                                                  'KOO                            ', & ! NAME(317)
                                                                                  'KOO_FULL                       ', & ! NAME(318)
                                                                                  'KOOs                           ', & ! NAME(319)
                                                                                  'KRL                            ', & ! NAME(320)
                                                                                  'KRR                            ', & ! NAME(321)
                                                                                  'KRRcb                          ', & ! NAME(322)
                                                                                  'KRRcbn                         ', & ! NAME(323)
                                                                                  'KRRcbs                         ', & ! NAME(324)
                                                                                  'KSF                            ', & ! NAME(325)
                                                                                  'KSFD                           ', & ! NAME(326)
                                                                                  'KSS                            ', & ! NAME(327)
                                                                                  'KSS_FULL                       ', & ! NAME(328)
                                                                                  'KSSe                           ', & ! NAME(329)
                                                                                  'KSSe_FULL                      ', & ! NAME(330)
                                                                                  'KXX                            ', & ! NAME(331)
                                                                                  'LABEL                          ', & ! NAME(332)
                                                                                  'LAPACK_S                       ', & ! NAME(333)
                                                                                  'LMN                            ', & ! NAME(334)
                                                                                  'LOAD_FACS                      ', & ! NAME(335)
                                                                                  'LOAD_SIDS                      ', & ! NAME(336)
                                                                                  'LOGICAL_VEC                    ', & ! NAME(337)
                                                                                  'LTM                            ', & ! NAME(338)
                                                                                  'MAA                            ', & ! NAME(339)
                                                                                  'MAA_FULL                       ', & ! NAME(340)
                                                                                  'MAO                            ', & ! NAME(341)
                                                                                  'MAO_FULL                       ', & ! NAME(342)
                                                                                  'MATANGLE                       ', & ! NAME(343)
                                                                                  'MATL                           ', & ! NAME(344)
                                                                                  'ME                             ', & ! NAME(345)
                                                                                  'MEFFMASS                       ', & ! NAME(346)
                                                                                  'MFF                            ', & ! NAME(347)
                                                                                  'MFF_FULL                       ', & ! NAME(348)
                                                                                  'MFS                            ', & ! NAME(349)
                                                                                  'MFS_FULL                       ', & ! NAME(350)
                                                                                  'MGG                            ', & ! NAME(351)
                                                                                  'MGGC                           ', & ! NAME(352)
                                                                                  'MGGE                           ', & ! NAME(353)
                                                                                  'MGGS                           ', & ! NAME(354)
                                                                                  'MLL                            ', & ! NAME(355)
                                                                                  'MLLn                           ', & ! NAME(356)
                                                                                  'MLLs                           ', & ! NAME(357)
                                                                                  'MLR                            ', & ! NAME(358)
                                                                                  'MMM                            ', & ! NAME(359)
                                                                                  'MMM_FULL                       ', & ! NAME(360)
                                                                                  'MMN                            ', & ! NAME(361)
                                                                                  'MNM                            ', & ! NAME(362)
                                                                                  'MNM_FULL                       ', & ! NAME(363)
                                                                                  'MNN                            ', & ! NAME(364)
                                                                                  'MNN_FULL                       ', & ! NAME(365)
                                                                                  'MODE_NUM                       ', & ! NAME(366)
                                                                                  'MOO                            ', & ! NAME(367)
                                                                                  'MOO_FULL                       ', & ! NAME(368)
                                                                                  'MPC_IND_GRIDS                  ', & ! NAME(369)
                                                                                  'MPC_SIDS                       ', & ! NAME(370)
                                                                                  'MPCADD_SIDS                    ', & ! NAME(371)
                                                                                  'MPCSETS                        ', & ! NAME(372)
                                                                                  'MPCSIDS                        ', & ! NAME(373)
                                                                                  'MPF0                           ', & ! NAME(374)
                                                                                  'MPFACTOR_N6                    ', & ! NAME(375)
                                                                                  'MPFACTOR_NR                    ', & ! NAME(376)
                                                                                  'MRL                            ', & ! NAME(377)
                                                                                  'MRN                            ', & ! NAME(378)
                                                                                  'MRR                            ', & ! NAME(379)
                                                                                  'MRRcb                          ', & ! NAME(380)
                                                                                  'MRRcbn                         ', & ! NAME(381)
                                                                                  'MSF                            ', & ! NAME(382)
                                                                                  'MSPRNT                         ', & ! NAME(383)
                                                                                  'MSS                            ', & ! NAME(384)
                                                                                  'MSS_FULL                       ', & ! NAME(385)
                                                                                  'MXX                            ', & ! NAME(386)
                                                                                  'MXXn                           ', & ! NAME(387)
                                                                                  'NL_SID                         ', & ! NAME(388)
                                                                                  'OELOUT                         ', & ! NAME(389)
                                                                                  'OFFDIS                         ', & ! NAME(390)
                                                                                  'OFFSET                         ', & ! NAME(391)
                                                                                  'OGEL                           ', & ! NAME(392)
                                                                                  'OGROUT                         ', & ! NAME(393)
                                                                                  'ONE_SET_ARRAY                  ', & ! NAME(394)
                                                                                  'OTM_ACCE                       ', & ! NAME(395)
                                                                                  'OTM_DISP                       ', & ! NAME(396)
                                                                                  'OTM_ELFE                       ', & ! NAME(397)
                                                                                  'OTM_ELFN                       ', & ! NAME(398)
                                                                                  'OTM_MPCF                       ', & ! NAME(399)
                                                                                  'OTM_SPCF                       ', & ! NAME(400)
                                                                                  'OTM_STRE                       ', & ! NAME(401)
                                                                                  'OTM_STRN                       ', & ! NAME(402)
                                                                                  'OU4_MAT_COL_GRD_COMP           ', & ! NAME(403)
                                                                                  'OU4_MAT_ROW_GRD_COMP           ', & ! NAME(404)
                                                                                  'PA                             ', & ! NAME(405)
                                                                                  'PA_FULL                        ', & ! NAME(406)
                                                                                  'PBAR                           ', & ! NAME(407)
                                                                                  'PBEAM                          ', & ! NAME(408)
                                                                                  'PBUSH                          ', & ! NAME(409)
                                                                                  'PCOMP                          ', & ! NAME(410)
                                                                                  'PDATA                          ', & ! NAME(411)
                                                                                  'PEB                            ', & ! NAME(412)
                                                                                  'PEG                            ', & ! NAME(413)
                                                                                  'PEL                            ', & ! NAME(414)
                                                                                  'PELAS                          ', & ! NAME(415)
                                                                                  'PF                             ', & ! NAME(416)
                                                                                  'PF_FULL                        ', & ! NAME(417)
                                                                                  'PF_TMP                         ', & ! NAME(418)
                                                                                  'PFYS                           ', & ! NAME(419)
                                                                                  'PFYS_FULL                      ', & ! NAME(420)
                                                                                  'PFYS1                          ', & ! NAME(421)
                                                                                  'PG                             ', & ! NAME(422)
                                                                                  'PG_COL                         ', & ! NAME(423)
                                                                                  'PHIXA                          ', & ! NAME(424)
                                                                                  'PHIXG_COL                      ', & ! NAME(425)
                                                                                  'PHIXGP_COL                     ', & ! NAME(426)
                                                                                  'PHIXL_COL                      ', & ! NAME(427)
                                                                                  'PHIXLP_COL                     ', & ! NAME(428)
                                                                                  'PHIXN_COL                      ', & ! NAME(429)
                                                                                  'PHIXNP_COL                     ', & ! NAME(430)
                                                                                  'PHIZG_FULL                     ', & ! NAME(431)
                                                                                  'PHIZL                          ', & ! NAME(432)
                                                                                  'PHIZL1                         ', & ! NAME(433)
                                                                                  'PHIZL1t                        ', & ! NAME(434)
                                                                                  'PHIZL2                         ', & ! NAME(435)
                                                                                  'PL                             ', & ! NAME(436)
                                                                                  'PL_COL                         ', & ! NAME(437)
                                                                                  'PLATEOFF                       ', & ! NAME(438)
                                                                                  'PLATETHICK                     ', & ! NAME(439)
                                                                                  'PM                             ', & ! NAME(440)
                                                                                  'PM_COL                         ', & ! NAME(441)
                                                                                  'PM_FULL                        ', & ! NAME(442)
                                                                                  'PMASS                          ', & ! NAME(443)
                                                                                  'PN                             ', & ! NAME(444)
                                                                                  'PN_FULL                        ', & ! NAME(445)
                                                                                  'PO                             ', & ! NAME(446)
                                                                                  'PO_FULL                        ', & ! NAME(447)
                                                                                  'POLY_FIT_ERR                   ', & ! NAME(448)
                                                                                  'POLY_FIT_ERR_INDEX             ', & ! NAME(449)
                                                                                  'PPE                            ', & ! NAME(450)
                                                                                  'PPNT                           ', & ! NAME(451)
                                                                                  'PR                             ', & ! NAME(452)
                                                                                  'PRESS                          ', & ! NAME(453)
                                                                                  'PLOAD4_3D_DATA                 ', & ! NAME(454)
                                                                                  'PRESS_SIDS                     ', & ! NAME(455)
                                                                                  'PROD                           ', & ! NAME(456)
                                                                                  'PS                             ', & ! NAME(457)
                                                                                  'PS_COL                         ', & ! NAME(458)
                                                                                  'PS_FULL                        ', & ! NAME(459)
                                                                                  'PSHEAR                         ', & ! NAME(460)
                                                                                  'PSHEL                          ', & ! NAME(461)
                                                                                  'PSOLID                         ', & ! NAME(462)
                                                                                  'PTE                            ', & ! NAME(463)
                                                                                  'PTYPE                          ', & ! NAME(464)
                                                                                  'PUSER1                         ', & ! NAME(465)
                                                                                  'PUSERIN                        ', & ! NAME(466)
                                                                                  'QGm_COL                        ', & ! NAME(467)
                                                                                  'QGr_COL                        ', & ! NAME(468)
                                                                                  'QGs_COL                        ', & ! NAME(469)
                                                                                  'QM_COL                         ', & ! NAME(470)
                                                                                  'QN_COL                         ', & ! NAME(471)
                                                                                  'QR_COL                         ', & ! NAME(472)
                                                                                  'QS_COL                         ', & ! NAME(473)
                                                                                  'QSYS                           ', & ! NAME(474)
                                                                                  'QSYS_COL                       ', & ! NAME(475)
                                                                                  'QSYS_FULL                      ', & ! NAME(476)
                                                                                  'RBGLOBAL_ASET                  ', & ! NAME(477)
                                                                                  'RBGLOBAL_FSET                  ', & ! NAME(478)
                                                                                  'RBGLOBAL_GSET                  ', & ! NAME(479)
                                                                                  'RBGLOBAL_LSET                  ', & ! NAME(480)
                                                                                  'RBGLOBAL_NSET                  ', & ! NAME(481)
                                                                                  'RCONM2                         ', & ! NAME(482)
                                                                                  'RCORD                          ', & ! NAME(483)
                                                                                  'REAL_VEC                       ', & ! NAME(484)
                                                                                  'RES                            ', & ! NAME(485)
                                                                                  'RESID                          ', & ! NAME(486)
                                                                                  'RFAC                           ', & ! NAME(487)
                                                                                  'RFORCE_SIDS                    ', & ! NAME(488)
                                                                                  'RGRID                          ', & ! NAME(489)
                                                                                  'RIGID_ELEM_IDS                 ', & ! NAME(490)
                                                                                  'RMATL                          ', & ! NAME(491)
                                                                                  'RMG                            ', & ! NAME(492)
                                                                                  'RMM                            ', & ! NAME(493)
                                                                                  'RMM_FULL                       ', & ! NAME(494)
                                                                                  'RMN                            ', & ! NAME(495)
                                                                                  'RPBAR                          ', & ! NAME(496)
                                                                                  'RPBEAM                         ', & ! NAME(497)
                                                                                  'RPBUSH                         ', & ! NAME(498)
                                                                                  'RPCOMP                         ', & ! NAME(499)
                                                                                  'RPELAS                         ', & ! NAME(500)
                                                                                  'RPMASS                         ', & ! NAME(501)
                                                                                  'RPROD                          ', & ! NAME(502)
                                                                                  'RPSHEAR                        ', & ! NAME(503)
                                                                                  'RPSHEL                         ', & ! NAME(504)
                                                                                  'RPUSER1                        ', & ! NAME(505)
                                                                                  'SC_ACCE                        ', & ! NAME(506)
                                                                                  'SC_DISP                        ', & ! NAME(507)
                                                                                  'SC_ELFE                        ', & ! NAME(508)
                                                                                  'SC_ELFN                        ', & ! NAME(509)
                                                                                  'SC_GPFO                        ', & ! NAME(510)
                                                                                  'SC_MPCF                        ', & ! NAME(511)
                                                                                  'SC_OLOA                        ', & ! NAME(512)
                                                                                  'SC_SPCF                        ', & ! NAME(513)
                                                                                  'SC_STRE                        ', & ! NAME(514)
                                                                                  'SC_STRN                        ', & ! NAME(515)
                                                                                  'SCNUM                          ', & ! NAME(516)
                                                                                  'SE1                            ', & ! NAME(517)
                                                                                  'SE2                            ', & ! NAME(518)
                                                                                  'SE3                            ', & ! NAME(519)
                                                                                  'SELECT                         ', & ! NAME(520)
                                                                                  'SEQ1                           ', & ! NAME(521)
                                                                                  'SEQ2                           ', & ! NAME(522)
                                                                                  'SETS_IDS                       ', & ! NAME(523)
                                                                                  'SLOAD_SIDS                     ', & ! NAME(524)
                                                                                  'SPC_SIDS                       ', & ! NAME(525)
                                                                                  'SPC1_SIDS                      ', & ! NAME(526)
                                                                                  'SPCSETS                        ', & ! NAME(527)
                                                                                  'SPCADD_SIDS                    ', & ! NAME(528)
                                                                                  'SPCSIDS                        ', & ! NAME(529)
                                                                                  'STE1                           ', & ! NAME(530)
                                                                                  'STE2                           ', & ! NAME(531)
                                                                                  'STE3                           ', & ! NAME(532)
                                                                                  'STF                            ', & ! NAME(533)
                                                                                  'STFCOL                         ', & ! NAME(534)
                                                                                  'STFKEY                         ', & ! NAME(535)
                                                                                  'STFPNT                         ', & ! NAME(536)
                                                                                  'STF3                           ', & ! NAME(537)
                                                                                  'STITLE                         ', & ! NAME(538)
                                                                                  'SUBLOD                         ', & ! NAME(539)
                                                                                  'SYS_LOAD                       ', & ! NAME(540)
                                                                                  'TDATA                          ', & ! NAME(541)
                                                                                  'TDOF                           ', & ! NAME(542)
                                                                                  'TDOF_ROW_START                 ', & ! NAME(543)
                                                                                  'TDOFI                          ', & ! NAME(544)
                                                                                  'TEMPLATE                       ', & ! NAME(545)
                                                                                  'TITLE                          ', & ! NAME(546)
                                                                                  'TN                             ', & ! NAME(547)
                                                                                  'TPNT                           ', & ! NAME(548)
                                                                                  'TR6_0                          ', & ! NAME(549)
                                                                                  'TR6_CG                         ', & ! NAME(550)
                                                                                  'TR6_MEFM                       ', & ! NAME(551)
                                                                                  'TSET                           ', & ! NAME(552)
                                                                                  'TXT_ACCE                       ', & ! NAME(553)
                                                                                  'TXT_DISP                       ', & ! NAME(554)
                                                                                  'TXT_ELFE                       ', & ! NAME(555)
                                                                                  'TXT_ELFN                       ', & ! NAME(556)
                                                                                  'TXT_MPCF                       ', & ! NAME(557)
                                                                                  'TXT_SPCF                       ', & ! NAME(558)
                                                                                  'TXT_STRE                       ', & ! NAME(559)
                                                                                  'TXT_STRN                       ', & ! NAME(560)
                                                                                  'UA_COL                         ', & ! NAME(561)
                                                                                  'UEB                            ', & ! NAME(562)
                                                                                  'UEG                            ', & ! NAME(563)
                                                                                  'UEL                            ', & ! NAME(564)
                                                                                  'UF_COL                         ', & ! NAME(565)
                                                                                  'UG_COL                         ', & ! NAME(566)
                                                                                  'UGG                            ', & ! NAME(567)
                                                                                  'UL_COL                         ', & ! NAME(568)
                                                                                  'UM_COL                         ', & ! NAME(569)
                                                                                  'UN_COL                         ', & ! NAME(570)
                                                                                  'UO_COL                         ', & ! NAME(571)
                                                                                  'UO0_COL                        ', & ! NAME(572)
                                                                                  'UR_COL                         ', & ! NAME(573)
                                                                                  'US_COL                         ', & ! NAME(574)
                                                                                  'USERIN_ACT_GRIDS               ', & ! NAME(575)
                                                                                  'USERIN_MAT_NAMES               ', & ! NAME(576)
                                                                                  'USET                           ', & ! NAME(577)
                                                                                  'VBAS                           ', & ! NAME(578)
                                                                                  'VVEC                           ', & ! NAME(579)
                                                                                  'WORKD                          ', & ! NAME(580)
                                                                                  'WORKL                          ', & ! NAME(581)
                                                                                  'XEB                            ', & ! NAME(582)
                                                                                  'XEL                            ', & ! NAME(583)
                                                                                  'XGL                            ', & ! NAME(584)
                                                                                  'YSe                            ' /) ! NAME(585)

      REAL(DOUBLE)                    :: ALLOCATED_ARRAY_MEM(NUM_ALLOC_ARRAYS) = (/(ZERO, II=1,NUM_ALLOC_ARRAYS)/)

      END MODULE ALLOCATED_ARRAY_DATA

